import sudo
import json


class ApprovalTestPlugin(sudo.Plugin):
    def __init__(self, plugin_options, **kwargs):
        id = sudo.options_as_dict(plugin_options).get("Id", "")
        super().__init__(plugin_options=plugin_options, **kwargs)
        self._id = "(APPROVAL {})".format(id)
        sudo.log_info("{} Constructed:".format(self._id))
        sudo.log_info(json.dumps(self.__dict__, indent=4, sort_keys=True))

    def __del__(self):
        sudo.log_info("{} Destructed successfully".format(self._id))

    def check(self, *args):
        sudo.log_info("{} Check was called with arguments: "
                      "{}".format(self._id, args))

    def show_version(self, *args):
        sudo.log_info("{} Show version was called with arguments: "
                      "{}".format(self._id, args))



import sudo
import signal
from os import path


class ReasonLoggerIOPlugin(sudo.Plugin):
    """
    An example sudo plugin demonstrating how to use the sudo conversation API.

    From the python plugin, you can ask something from the user using the
    "sudo.conv" function. It expects one or more "sudo.ConvMessage" instances
    which specifies how the interaction has to look like.

    sudo.ConvMessage has the following fields (see help(sudo.ConvMessage)):
        msg_type: int  Specifies the type of the conversation.
                       See sudo.CONV.* constants below.
        timeout: int   The maximum amount of time for the conversation
                       in seconds. After the timeout exceeds, the "sudo.conv"
                       function will raise sudo.ConversationInterrupted
                       exception.
        msg: str       The message to display for the user.

    To specify the conversion type you can use the following constants:
        sudo.CONV.PROMPT_ECHO_OFF
        sudo.CONV.PROMPT_ECHO_ON
        sudo.CONV.ERROR_MSG
        sudo.CONV.INFO_MSG
        sudo.CONV.PROMPT_MASK
        sudo.CONV.PROMPT_ECHO_OK
        sudo.CONV.PREFER_TTY
    """

    def open(self, argv, command_info):
        try:
            conv_timeout = 120  # in seconds
            sudo.log_info("Please provide your reason "
                          "for executing {}".format(argv))

            # We ask two questions, the second is not visible on screen,
            # so the user can hide a hidden message in case of criminals are
            # forcing him for running the command.
            # You can either specify the arguments in strict order (timeout
            # being optional), or use named arguments.
            message1 = sudo.ConvMessage(sudo.CONV.PROMPT_ECHO_ON,
                                        "Reason: ",
                                        conv_timeout)
            message2 = sudo.ConvMessage(msg="Secret reason: ",
                                        timeout=conv_timeout,
                                        msg_type=sudo.CONV.PROMPT_MASK)
            reply1, reply2 = sudo.conv(message1, message2,
                                       on_suspend=self.on_conversation_suspend,
                                       on_resume=self.on_conversation_resume)

            with open(self._log_file_path(), "a") as file:
                print("Executed", ' '.join(argv), file=file)
                print("Reason:", reply1, file=file)
                print("Hidden reason:", reply2, file=file)

        except sudo.ConversationInterrupted:
            sudo.log_error("You did not answer in time")
            return sudo.RC.REJECT

    def on_conversation_suspend(self, signum):
        # This is just an example of how to do something on conversation
        # suspend. You can skip specifying 'on_suspend' argument if there
        # is no need
        sudo.log_info("conversation suspend: signal",
                      self._signal_name(signum))

    def on_conversation_resume(self, signum):
        # This is just an example of how to do something on conversation
        # resume. You can skip specifying 'on_resume' argument if there
        # is no need
        sudo.log_info("conversation resume: signal was",
                      self._signal_name(signum))

    # helper functions:
    if hasattr(signal, "Signals"):
        @classmethod
        def _signal_name(cls, signum: int):
            try:
                return signal.Signals(signum).name
            except Exception:
                return "{}".format(signum)
    else:
        @classmethod
        def _signal_name(cls, signum: int):
            for n, v in sorted(signal.__dict__.items()):
                if v != signum:
                    continue
                if n.startswith("SIG") and not n.startswith("SIG_"):
                    return n
            return "{}".format(signum)

    def _log_file_path(self):
        options_dict = sudo.options_as_dict(self.plugin_options)
        log_path = options_dict.get("LogPath", "/tmp")
        return path.join(log_path, "sudo_reasons.txt")



import sudo

import logging


class DebugDemoPlugin(sudo.Plugin):
    """
    An example sudo plugin demonstrating the debugging capabilities.

    You can install it as an extra IO plugin for example by adding the
    following line to sudo.conf:
        Plugin python_io python_plugin.so \
            ModulePath=<path>/example_debugging.py \
            ClassName=DebugDemoPlugin

    To see the plugin's debug output, use the following line in sudo.conf:
        Debug python_plugin.so \
            /var/log/sudo_python_debug plugin@trace,c_calls@trace
            ^                          ^-- the options for the logging
            ^----- the output will be placed here

    The options for the logging is in format of multiple "subsystem@level"
    separated by commas (",").
    The most interesting subsystems are:
        plugin      Shows each call of sudo.debug API in the log
      - py_calls    Logs whenever a C function calls into the python module.
                    (For example calling this __init__ function.)
        c_calls     Logs whenever python calls into a C sudo API function

    You can also specify "all" as subsystem name to get the debug messages of
    all subsystems.

    Other subsystems available:
        internal    logs internal functions of the python language wrapper
        sudo_cb     logs when sudo calls into its plugin API
        load        logs python plugin loading / unloading

    Log levels
        crit      sudo.DEBUG.CRIT       --> only critical messages
        err       sudo.DEBUG.ERROR
        warn      sudo.DEBUG.WARN
        notice    sudo.DEBUG.NOTICE
        diag      sudo.DEBUG.DIAG
        info      sudo.DEBUG.INFO
        trace     sudo.DEBUG.TRACE
        debug     sudo.DEBUG.DEBUG      --> very extreme verbose debugging

    See the sudo.conf manual for more details ("man sudo.conf").

    """

    def __init__(self, plugin_options, **kwargs):
        # Specify: "py_calls@info" debug option to show the call to this
        # constructor and the arguments passed in

        # Specifying "plugin@err" debug option will show this message
        # (or any more verbose level)
        sudo.debug(sudo.DEBUG.ERROR, "My demo purpose plugin shows "
                   "this ERROR level debug message")

        # Specifying "plugin@info" debug option will show this message
        # (or any more verbose level)
        sudo.debug(sudo.DEBUG.INFO, "My demo purpose plugin shows "
                   "this INFO level debug message")

        # You can also use python log system, because sudo sets its log handler
        # on the root logger.
        # Note that the level of python logging is separate than the one set in
        # sudo.conf. If using the python logger, each will have effect.
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        logger.error("Python log system shows this ERROR level debug message")
        logger.info("Python log system shows this INFO level debug message")

        # If you raise the level to info or below, the call of the debug
        # will also be logged.
        # An example output you will see in the debug log file:
        #   Dec  5 15:19:19 sudo[123040] __init__ @ /.../example_debugging.py:54 debugs:
        #   Dec  5 15:19:19 sudo[123040] My demo purpose plugin shows this ERROR level debug message

        # Specify: "c_calls@diag" debug option to show this call and its
        # arguments. If you specify info debug level instead ("c_calls@info"),
        # you will also see the python function and line from which you called
        # the 'options_as_dict' function.
        self.plugin_options = sudo.options_as_dict(plugin_options)



import sudo


class SudoGroupPlugin(sudo.Plugin):
    """Example sudo input/output plugin

    Demonstrates how to use the sudo group plugin API. Typing annotations are
    just here for the help on the syntax (requires python >= 3.5).

    On detailed description of the functions refer to sudo_plugin manual (man
    sudo_plugin).

    Most functions can express error or reject through their "int" return value
    as documented in the manual. The sudo module also has constants for these:
        sudo.RC.ACCEPT / sudo.RC.OK  1
        sudo.RC.REJECT               0
        sudo.RC.ERROR               -1
        sudo.RC.USAGE_ERROR         -2

    If the plugin encounters an error, instead of just returning sudo.RC.ERROR
    result code it can also add a message describing the problem.
    This can be done by raising the special exception:
        raise sudo.PluginError("Message")
    This added message will be used by the audit plugins.

    If the function returns "None" (for example does not call return), it will
    be considered sudo.RC.OK. If an exception other than sudo.PluginError is
    raised, its backtrace will be shown to the user and the plugin function
    returns sudo.RC.ERROR. If that is not acceptable, catch it.
    """

    # -- Plugin API functions --
    def query(self, user: str, group: str, user_pwd: tuple):
        """Query if user is part of the specified group.

        Beware that user_pwd can be None if user is not present in the password
        database. Otherwise it is a tuple convertible to pwd.struct_passwd.
        """
        hardcoded_user_groups = {
            "testgroup": ["testuser1", "testuser2"],
            "mygroup": ["test"]
        }

        group_has_user = user in hardcoded_user_groups.get(group, [])
        return sudo.RC.ACCEPT if group_has_user else sudo.RC.REJECT
